library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity permtonum is
	port(
			clk		: in std_logic;
			data     : in std_logic_vector(4 downto 0);
			addr     : in std_logic_vector(1 downto 0);
			q        : out std_logic_vector(4 downto 0)
	);
end permtonum;

architecture implementation of permtonum is

  signal latch0     : std_logic_vector(4 downto 0);
  signal latch1     : std_logic_vector(4 downto 0);
  signal latch2     : std_logic_vector(4 downto 0);
  signal latch3     : std_logic_vector(4 downto 0);
  signal latch0_en  : std_logic;
  signal latch1_en  : std_logic;
  signal latch2_en  : std_logic;
  signal latch3_en  : std_logic;
  signal comp       : std_logic_vector(5 downto 0);
  
begin
  
  latch0_en <= clk and not addr(1) and not addr(0);
  latch1_en <= clk and not addr(1) and     addr(0);
  latch2_en <= clk and     addr(1) and not addr(0);
  latch3_en <= clk and     addr(1) and     addr(0);
  
  process (latch0_en, data)
  begin
    if latch0_en = '1' then
      latch0 <= data;
    end if;
  end process;

  process (latch1_en, data)
  begin
    if latch1_en = '1' then
      latch1 <= data;
	 end if;
  end process;

  process (latch2_en, data)
  begin
    if latch2_en = '1' then
      latch2 <= data;
	 end if;
  end process;

  process (latch3_en, data)
  begin
    if latch3_en = '1' then
      latch3 <= data;
	 end if;
  end process;
 
  process (latch1, latch0)
  begin
    if latch1 < latch0 then
      comp(5) <= '1';
	 else
	   comp(5) <= '0';
    end if;
  end process; 

  process (latch2, latch0)
  begin
    if latch2 < latch0 then
      comp(4) <= '1';
	 else
	   comp(4) <= '0';
    end if;
  end process;    
  
  process (latch3, latch0)
  begin
    if latch3 < latch0 then
      comp(3) <= '1';
	 else
	   comp(3) <= '0';
    end if;
  end process;  
  
  process (latch2, latch1)
  begin
    if latch2 < latch1 then
      comp(2) <= '1';
	 else
	   comp(2) <= '0';
    end if;
  end process;  
  
  process (latch3, latch1)
  begin
    if latch3 < latch1 then
      comp(1) <= '1';
	 else
	   comp(1) <= '0';
    end if;
  end process;
  
  process (latch3, latch2)
  begin
    if latch3 < latch2 then
      comp(0) <= '1';
	 else
	   comp(0) <= '0';
    end if;
  end process;  
 
  ROM: entity work.single_port_rom port map(
	  
    clk  => clk,
    addr => to_integer(unsigned(comp)),
    q    => q

  );
  
end implementation;
