-- Quartus II VHDL Template
-- Single-Port ROM

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity single_port_rom is

	generic 
	(
		DATA_WIDTH : natural := 5;
		ADDR_WIDTH : natural := 6
	);

	port 
	(
		clk	: in std_logic;
		addr	: in natural range 0 to 2**ADDR_WIDTH - 1;
		q		: out std_logic_vector((DATA_WIDTH -1) downto 0)
	);

end entity;

architecture rtl of single_port_rom is

	-- Build a 2-D array type for the RoM
	subtype word_t is std_logic_vector((DATA_WIDTH-1) downto 0);
	type memory_t is array(2**ADDR_WIDTH-1 downto 0) of word_t;

	function init_rom
		return memory_t is 
		variable tmp : memory_t := (others => (others => '0'));
	begin 
		tmp(0)  := "00000"; tmp(16) := "00110"; tmp(32) := "00110"; tmp(48) := "01100";
		tmp(1)  := "00001"; tmp(17) := "00111"; tmp(33) := "00111"; tmp(49) := "01101";
		tmp(2)  := "00010"; tmp(18) := "01000"; tmp(34) := "01000"; tmp(50) := "01110";
		tmp(3)  := "00011"; tmp(19) := "01001"; tmp(35) := "01001"; tmp(51) := "01111";
		tmp(4)  := "00010"; tmp(20) := "01000"; tmp(36) := "01000"; tmp(52) := "01110";
		tmp(5)  := "00011"; tmp(21) := "01001"; tmp(37) := "01001"; tmp(53) := "01111";
		tmp(6)  := "00100"; tmp(22) := "01010"; tmp(38) := "01010"; tmp(54) := "10000";
		tmp(7)  := "00101"; tmp(23) := "01011"; tmp(39) := "01011"; tmp(55) := "10001";
		tmp(8)  := "00110"; tmp(24) := "01100"; tmp(40) := "01100"; tmp(56) := "10010";
		tmp(9)  := "00111"; tmp(25) := "01101"; tmp(41) := "01101"; tmp(57) := "10011";
		tmp(10) := "01000"; tmp(26) := "01110"; tmp(42) := "01110"; tmp(58) := "10100";
		tmp(11) := "01001"; tmp(27) := "01111"; tmp(43) := "01111"; tmp(59) := "10101";
		tmp(12) := "01000"; tmp(28) := "01110"; tmp(44) := "01110"; tmp(60) := "10100";
		tmp(13) := "01001"; tmp(29) := "01111"; tmp(45) := "01111"; tmp(61) := "10101";
		tmp(14) := "01010"; tmp(30) := "10000"; tmp(46) := "10000"; tmp(62) := "10110";
		tmp(15) := "01011"; tmp(31) := "10001"; tmp(47) := "10001"; tmp(63) := "10111";
		return tmp;
	end init_rom;	 

	-- Declare the ROM signal and specify a default value.	Quartus II
	-- will create a memory initialization file (.mif) based on the 
	-- default value.
	signal rom : memory_t := init_rom;

begin

	process(clk)
	begin
	if(rising_edge(clk)) then
		q <= rom(addr);
	end if;
	end process;

end rtl;
